package org.infinispan.remoting.responses;

import org.infinispan.marshall.Marshalls;
import org.infinispan.marshall.Ids;

import java.io.ObjectOutput;
import java.io.IOException;
import java.io.ObjectInput;

/**
 * An unsure response - used with Dist - essentially asks the caller to check the next response from the next node since
 * the sender is in a state of flux (probably in the middle of rebalancing)
 *
 * @author Manik Surtani
 * @since 4.0
 */
public class UnsureResponse extends ValidResponse {
   public static final UnsureResponse INSTANCE = new UnsureResponse();
   public boolean isSuccessful() {
      return false;
   }

   @Marshalls(typeClasses = UnsureResponse.class, id = Ids.UNSURE_RESPONSE)
   public static class Externalizer implements org.infinispan.marshall.Externalizer<UnsureResponse> {
      public void writeObject(ObjectOutput output, UnsureResponse subject) throws IOException {
      }

      public UnsureResponse readObject(ObjectInput input) throws IOException, ClassNotFoundException {
         return INSTANCE;
      }
   }   
}
